@@ -146,6 +146,12 @@ struct cabextract_args args = {
 
 #if HAVE_ICONV
 iconv_t converter = NULL;
+/* names for the UTF-8 charset recognised by different iconv_open()s */
+char *utf8_names[] = {
+    "UTF-8", /* glibc, libiconv, FreeBSD, Solaris, not newlib or HPUX */
+    "UTF8",  /* glibc, libiconv (< 1.13), newlib, HPUX */
+    "UTF_8", /* newlib, Solaris */
+};
 #endif
 
 /** A special filename. Extracting to this filename will send the output
@@ -220,27 +226,20 @@ static struct mspack_system cabextract_system = {
 int main(int argc, char *argv[]) {
   int i, err;
 
-  /* names for the UTF-8 charset recognised by different iconv_open()s */
-  char *utf8_names[] = {
-      "UTF-8", /* glibc, libiconv, FreeBSD, Solaris, not newlib or HPUX */
-      "UTF8",  /* glibc, libiconv (< 1.13), newlib, HPUX */
-      "UTF_8", /* newlib, Solaris */
+  /* attempt to set a UTF8-based locale, so that tolower()/towlower()
+   * in create_output_name() lowercase more than just A-Z in ASCII.
+   *
+   * We don't attempt to pick up the system default locale, "",
+   * because it might not be compatible with ASCII/ISO-8859-1/Unicode
+   * character codes and would mess up lowercased filenames
+   */
+  char *locales[] = {
+      "C.UTF-8", /* https://sourceware.org/glibc/wiki/Proposals/C.UTF-8 */
+      "en_US.UTF-8", "en_GB.UTF8", "de_DE.UTF-8", "UTF-8", "UTF8"
   };
-
-   /* attempt to set a UTF8-based locale, so that tolower()/towlower()
-    * in create_output_name() lowercase more than just A-Z in ASCII.
-    *
-    * We don't attempt to pick up the system default locale, "",
-    * because it might not be compatible with ASCII/ISO-8859-1/Unicode
-    * character codes and would mess up lowercased filenames
-    */
-   char *locales[] = {
-       "C.UTF-8", /* https://sourceware.org/glibc/wiki/Proposals/C.UTF-8 */
-       "en_US.UTF-8", "en_GB.UTF8", "de_DE.UTF-8", "UTF-8", "UTF8"
-   };
-   for (i = 0; i < (sizeof(locales)/sizeof(*locales)); i++) {
+  for (i = 0; i < (sizeof(locales)/sizeof(*locales)); i++) {
       if (setlocale(LC_CTYPE, locales[i])) break;
-   }
+  }
 
   /* parse options */
   while ((i = getopt_long(argc, argv, OPTSTRING, optlist, NULL)) != -1) {
@@ -1036,8 +1035,7 @@ static char *convert_filename(char *name) {
     /* worst case: all characters expand from 1 to 4 bytes */
     size_t ilen = strlen(name) + 1, olen = ilen * 4;
     ICONV_CONST char *i = name;
-    char *newname = malloc(olen);
-    unsigned char *o = (unsigned char *) newname;
+    char *newname = malloc(olen), *o = newname;
 
     if (!newname) {
         fprintf(stderr, "WARNING: out of memory converting filename\n");
@@ -1046,11 +1044,11 @@ static char *convert_filename(char *name) {
 
     /* convert filename to UTF8 */
     iconv(converter, NULL, NULL, NULL, NULL);
-    while (iconv(converter, &i, &ilen, (char **) &o, &olen) == (size_t) -1) {
+    while (iconv(converter, &i, &ilen, &o, &olen) == (size_t) -1) {
         if (errno == EILSEQ || errno == EINVAL) {
             /* invalid or incomplete multibyte sequence: skip it */
             i++; ilen--;
-            *o++ = 0xEF; *o++ = 0xBF; *o++ = 0xBD; olen += 3;
+            memcpy(o, "\xEF\xBF\xBD", 3); o += 3; olen += 3;
         }
         else /* E2BIG: should be impossible to get here */ {
             free(newname);
@@ -1227,8 +1225,10 @@ static int ensure_filepath(char *path, int n) {
       /* in the archive-determined part of the path and not keeping symlinks:
        * use lstat() and delete symlinks if found */
       ok = (lstat(path, &st_buf) == 0);
-      if (ok && (st_buf.st_mode & S_IFMT) == S_IFLNK) unlink(path);
-      ok = S_ISDIR(st_buf.st_mode);
+      if (ok) {
+        if ((st_buf.st_mode & S_IFMT) == S_IFLNK) unlink(path);
+        ok = S_ISDIR(st_buf.st_mode);
+      }
     }
     if (!ok) ok = (mkdir(path, 0777 & ~user_umask) == 0);
     *p = '/';
